/*   AM-STD for ICP-PCI Kernel module
 *   free public release
 *
 *   Modified by: Rolando E.Cantarutti    email: rcantarutti@noao.edu
 *                Cerro Tololo Interamerican Observatory
 *                AURA INC.
 *
 *      referring to: AM-CLR developed by the following people
 *
 *   Werner Jaeger        email: e9026603@student.tuwien.ac.at
 *   Wilfried Kubinger    email: wk@flexaut.tuwien.ac.at
 *
 *   Institute of Flexible Automation
 *   University of Technology Vienna
 *   Austria, Europe
 *
 *   amcmpReg.c  v2.0  1999/09/09
 */

/* Change History:

 */

/*
 * Defines:
 */

#define DEBUG_ICPCI 1
/*#define DEBUG_ICPCI_REG 1 */

#define	ICP_AMCC_READ	0	/* current op is read op */
#define ICP_AMCC_WRITE	1	/* current op is write op */
#define	ICP_AMCC_EVEN	0	/* latch status even read/write */
#define ICP_AMCC_ODD	1	/* latch status odd read/write */

/*
 * Includes:
 */

extern unsigned long loops_per_sec;	/* for delay.h */

#include "icpciOS.h"    /* os specific stuff */
#include "icpciExt.h"   /* offical driver interface */
#include "icpciInt.h"   /* driver internals */
#include "amcmpReg.h"   /* register definitions AM-DIG */

/*
 * Globals:
 */

extern struct
icp_global_t    icp;		/* main driver structure */

#if (DEBUG_ICPCI || DEBUG_ICPCI_REG)
const char  *cmp_reg[] = { "modid   ",
			   "mxctl   ",
			   "scl     ",
			   "sda     ",
			   "nptrig  ",
                           "npswtrig",
			   "reserved",
			   "reserved"
			};

const char  *cmp_bt829a_reg[] = { "status  ",
                                  "iform   ",
                                  "tdec    ",
                                  "crop    ",
                                  "vdly_lo ",
                                  "vact_lo ",
                                  "hdly_lo ",
                                  "hact_lo ",
                                  "hscal_hi",
                                  "hscal_lo",
                                  "bright  ",
                                  "control ",
                                  "con_lo  ",
                                  "sat_u_lo",
                                  "sat_v_lo",
                                  "hue     ",
                                  "scloop  ",
                                  "reserved",
                                  "oform   ",
                                  "vscal_hi",
                                  "vscal_lo",
                                  "reserved",
                                  "reserved",
                                  "idcode  ",
                                  "adly    ",
                                  "bdly    ",
                                  "adc     ",
                                  "vtc     ",
                                  "reserved",
                                  "reserved",
                                  "reserved",
                                  "sreset  "
			};
#endif /* DEBUG_ICPCI_REG */

/*
 * Prototypes:
 */

/*
 * General Read Handler for AM-STD Registers
 */

/* 16 bit register access with mask and range checking */

static u_short
cmp_read_mm (board, offset, mask, min, max)
     
  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_short offset;        /* offset to register base address pointer */
  u_short mask;          /* read mask for register */
  u_short min;           /* minimal value for range checking */
  u_short max;           /* maximal value for range checking */

{
  u_short tmp_val = 0;

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  cmp_read_mm %s, %#06X &(%#06X) >%#06X <%#06X\n",
         board, cmp_reg[offset], tmp_val, mask, min, max);
#endif

  return (tmp_val);
}

/* 16 bit register 1 (or more) bit access */

static u_char
cmp_read_ms (board, offset, mask, shift)

  u_char  board;		/* number of IC-PCI board attached to the computer */
  u_char  offset;	/* offset to register base address pointer */
  u_char  shift;		/* number of bits to shift right */
  u_short mask;		/* read mask for register */
  
{
  u_short tmp_val = 0;

  tmp_val = icp_readw (icp.map[board]->v_mod_base + offset);

  tmp_val &= mask;
  tmp_val >>= shift;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  cmp-readw  %s, %#1X &(%#06X) >>%2d\n",
	     board, cmp_reg[offset], tmp_val, mask, shift);
#endif

  return ((u_char)tmp_val);
}

/*
 * General Write Handler for AM-DIG Registers
 */

/* 16 bit register access with mask and range checking */

static int
cmp_write_mm (board, offset, mask, min, max, value)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_short offset;        /* offset to register base address pointer */
  u_short mask;          /* read mask for register */
  u_short min;           /* minimal value for range checking */
  u_short max;           /* maximal value for range checking */
  u_short value;         /* value to set to */

{
  u_short tmp_val = 0;

  if (value & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  if (value < min || value > max) {
    icp.errno[board] = -EICP_RANGE;  
    return (ERROR);
  }

  tmp_val = value & mask;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  cmp_write_mm %s, %#06X &(%#06X) >%#06X <%#06X\n",
             board, cmp_reg[offset], tmp_val, mask, min, max);
#endif

  icp_writew (tmp_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}

/* 16 bit register 1 (or more) bit access */

static int
cmp_write_ms (board, offset, mask, shift, value)

  u_char  board;         /* number of IC-PCI board attached to the computer */
  u_short offset;        /* offset to register base address pointer */
  u_short mask;          /* read mask for register */
  u_char  shift;         /* number of bits to shift right */
  u_char  value;         /* value to set to */  
  
{
  u_short read_val = 0;
  u_short write_val = 0;
  u_short tmp_val = 0;

  if (offset > CMP_REG_SIZE) {
    icp.errno[board] = -EICP_NOSUPPORT;    
    return (ERROR);
  }

  tmp_val = (u_short)value << shift;

  if (tmp_val & ~mask) {
    icp.errno[board] = -EICP_MASK;
    return (ERROR);
  }

  read_val = icp_readw (icp.map[board]->v_mod_base + offset);

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:  %s, %#06X->\n", board,
             cmp_reg[offset], read_val);
#endif

  read_val &= ~mask;
  write_val = read_val | tmp_val;

#ifdef DEBUG_ICPCI_REG
  icp_printf("icpci[%d]:     %#06X\n", board, write_val);
#endif

  icp_writew (write_val, icp.map[board]->v_mod_base + offset);

  return (OK);
}
  
/*
 * Special Read Handler for AM-STD Registers
 */

u_char
cmp_get_rev (u_char board) {

  return (cmp_read_ms (board,
		      CMP_MODID_OFFSET,
		      CMP_REV_MASK,
		      CMP_REV_SHIFT));
}

u_char
cmp_get_brdid (u_char board) {

  return (cmp_read_ms (board,
		      CMP_MODID_OFFSET,
		      CMP_BRDID_MASK,
		      CMP_BRDID_SHIFT));
}

u_char
cmp_get_sdarbk (u_char board) {

  return (cmp_read_ms (board,
                       CMP_SDA_OFFSET,
                       CMP_SDARBK_MASK,
                       CMP_SDARBK_SHIFT));
}
                

int
cmp_set_scl0 (u_char board, u_char value) {

  return (cmp_write_ms (board,
                       CMP_SCL_OFFSET,
                       CMP_SCL0_MASK,
                       CMP_SCL0_SHIFT,
                       value));
}

int
cmp_set_sclen (u_char board, u_char value) {

  return (cmp_write_ms (board,
                       CMP_SCL_OFFSET,
                       CMP_SCLEN_MASK,
                       CMP_SCLEN_SHIFT,
                       value));
}

int
cmp_set_sda0 (u_char board, u_char value) {

  return (cmp_write_ms (board,
                       CMP_SDA_OFFSET,
                       CMP_SDA0_MASK,
                       CMP_SDA0_SHIFT,
                       value));
}

int
cmp_set_sdaen (u_char board, u_char value) {

  return (cmp_write_ms (board,
                       CMP_SDA_OFFSET,
                       CMP_SDAEN_MASK,
                       CMP_SDAEN_SHIFT,
                       value));
}

int
cmp_set_ntrgen (u_char board, u_char value) {

  return (cmp_write_ms (board,
                        CMP_NPTRIG_OFFSET,
                        CMP_NTRGEN_MASK,
                        CMP_NTRGEN_SHIFT,
                        value));
}

u_char
cmp_i2c_read (u_char board, u_char address) {

  u_char baddress, tmp_val;
  int count;

  /* START CONDITION */

  /* enable serial clock and set SCL */
  cmp_set_sclen (board, CMP_SCLEN_ENABLE);
  cmp_set_scl0 (board, CMP_SCL0_HIGH);

  /* enable serial data and set SDA */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);
  cmp_set_sda0 (board, CMP_SDA0_HIGH);
  udelay (5);		/* bus free time between STOP and START */

  /* SDA goes low while SCL is high to signal the START condition */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (4);		/* hold time START condition */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* data hold time */

  /* SLAVE ADDRESS + DATA DIRECTION BIT 0 */

  baddress = CMP_BT829A_W_ADDRESS;

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (baddress >> (7 - count)) & 1);
    udelay (1);		/* data set-up time */
   
    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* RECEIVE ACKNOWLEDGE */

  /* dummy low state of SDA line while receiving acknowledge */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);          /* data set-up time */

  /* disable the SDA driver to receive acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);

  /* SCL goes high to generate acknowledge-related clock */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* high period of the SCL clock */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* low period of the SCL clock */

  /* re-enable the SDA driver after receiving acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);
 
  /* check for acknowledge */
  if (cmp_get_sdarbk (board) != 0)
    return (ERROR);

  /* SUB-ADDRESS */

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (address >> (7 - count)) & 1);
    udelay (1);         /* data set-up time */
   
    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* REPEATED START CONDITION */

  cmp_set_sda0 (board, CMP_SDA0_HIGH);
  udelay (1);		/* data set-up time */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (5);          /* setup time for a repeated START condition */

  /* SDA goes low while SCL is high to signal the START condition */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (4);		/* hold time START condition */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);          /* data hold time */

  /* READ CYCLE: SLAVE ADDRESS PLUS READ DIRECTION BIT */

  baddress = CMP_BT829A_R_ADDRESS;

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (baddress >> (7 - count)) & 1);
    udelay (1);         /* data set-up time */

    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* RECEIVE AKNOWLEDGE */

  /* dummy low state of SDA line while receiving acknowledge */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);           /* data set-up time */
  
  /* disable the SDA driver to receive acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);

  /* SCL goes high to generate acknowledge-related clock */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* high period of the SCL clock */
  
  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* low period of the SCL clock */

  /* check for acknowledge */
  if (cmp_get_sdarbk (board) != 0)
    return (ERROR);

  /* RECEIVE DATA */

  tmp_val = 0;
  for (count=0; count < 8; count ++) {

    /* SCL goes high to generate data bit clock */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);		/* low period of the SCL clock */

    tmp_val = tmp_val | (cmp_get_sdarbk (board) << (7 - count));
  }

  /* STOP CONDITION */

  /* re-enable the SDA driver to indicate the STOP condition */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);

  /* SDA returns from its dummy state to zero */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);          /* data set-up time */

  /* SCL goes high to generate stop condition */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* setup time for STOP condition */

  /* SDA goes high while SCL is high to signal the STOP condition */
  cmp_set_sda0 (board, CMP_SDA0_HIGH);

  /* disable SCL and SDA */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);
  cmp_set_sclen (board, CMP_SCLEN_DISABLE);

  return (tmp_val);
}

int
cmp_i2c_write (u_char board, u_char address, u_char value) {

  u_char baddress = CMP_BT829A_W_ADDRESS;
  int count;

  /* START CONDITION */

  /* enable serial clock and set SCL */
  cmp_set_sclen (board, CMP_SCLEN_ENABLE);
  cmp_set_scl0 (board, CMP_SCL0_HIGH);

  /* enable serial data and set SDA */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);
  cmp_set_sda0 (board, CMP_SDA0_HIGH);
  udelay (5);           /* bus free time between STOP and START */

 /* SDA goes low while SCL is high to signal the START condition */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (4);		/* hold time START condition */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);          /* data hold time */

  /* SLAVE ADDRESS + DATA DIRECTION BIT 0 */

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (baddress >> (7 - count)) & 1);
    udelay (1);		/* data set-up time */
   
    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* RECEIVE  ACKNOWLEDGE */

  /* dummy low state of the SDA line while receiving acknowledge */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);           /* data set-up time */

  /* disable the SDA driver to receive acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);

  /* SCL goes high to generate acknowledge-related clock */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* high period of the SCL clock */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* low period of the SCL clock */

  /* re-enable the SDA driver after receiving acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);
 
  /* check for acknowledge */
  if (cmp_get_sdarbk (board) != 0)
    return (ERROR);

  /* SUB-ADDRESS */

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (address >> (7 - count)) & 1);
    udelay (1);         /* data set-up time */
   
    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* READ THE ACKNOWLEDGE */

  /* dummy state of the SDA line while reading aknowledge */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);           /* data set-up time */

  /* disable the SDA driver to receive acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);

  /* SCL goes high to generate acknowledge-related clock */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* high period of the SCL clock */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* low period of the SCL clock */

  /* re-enable the SDA driver after receiving acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);

  /* check for acknowledge */
  if (cmp_get_sdarbk (board) != 0)
    return (ERROR);

  /* DATA BYTE */

  for (count=0; count < 8; count++) {

    /* change data */
    cmp_set_sda0 (board, (value >> (7 - count)) & 1);
    udelay (1);         /* data set-up time */
   
    /* SCL goes high to signal valid data on the SDA line */
    cmp_set_scl0 (board, CMP_SCL0_HIGH);
    udelay (4);          /* high period of the SCL clock */

    /* SCL goes low to allow the change of data */
    cmp_set_scl0 (board, CMP_SCL0_LOW);
    udelay (5);          /* data hold time */
  }

  /* READ THE ACKNOWLEDGE */

  /* dummy state of the SDA line while reading acknowledge */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (1);          /* data set-up time */

  /* disable the SDA driver to receive acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);

  /* SCL goes high to generate acknowledge-related clock */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* high period of the SCL clock */

  /* SCL goes low to allow the change of data */
  cmp_set_scl0 (board, CMP_SCL0_LOW);
  udelay (5);		/* low period of the SCL clock */

  /* re-enable the SDA driver after receiving acknowledge */
  cmp_set_sdaen (board, CMP_SDAEN_ENABLE);

  /* check for acknowledge */
  if (cmp_get_sdarbk (board) != 0)
    return (ERROR);

  /* STOP CONDITION */

  /* SDA returns from its dummy state to zero */
  cmp_set_sda0 (board, CMP_SDA0_LOW);
  udelay (5);          /* low period of the SCL clock */

  /* SCL goes high to generate stop condition */
  cmp_set_scl0 (board, CMP_SCL0_HIGH);
  udelay (4);          /* setup time for STOP condition */

  /* SDA goes high while SCL is high to signal the STOP condition */
  cmp_set_sda0 (board, CMP_SDA0_HIGH);
 
  /* disable SCL and SDA */
  cmp_set_sdaen (board, CMP_SDAEN_DISABLE);
  cmp_set_sclen (board, CMP_SCLEN_DISABLE);

  return (OK);
}

u_char
cmp_get_status (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_STATUS_ADDRESS));
}

u_char
cmp_get_iform (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_IFORM_ADDRESS));
}

u_char
cmp_get_tdec (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_TDEC_ADDRESS));
}

u_char
cmp_get_crop (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_CROP_ADDRESS));
}

u_char
cmp_get_vdly_lo (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_VDLY_LO_ADDRESS));
}

u_short
cmp_get_vdelay (u_char board) {
 
  u_short tmp_val;
 
  tmp_val = (u_short) cmp_get_crop (board);
  tmp_val = (tmp_val & CMP_BT829A_VDLY_MASK) >> CMPBT829A_VDLY_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_vdly_lo (board);
 
  return (tmp_val);
}

u_char
cmp_get_vact_lo (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_VACT_LO_ADDRESS));
}

u_short
cmp_get_vactive (u_char board) {

  u_short tmp_val;

  tmp_val = (u_short) cmp_get_crop (board);
  tmp_val = (tmp_val & CMP_BT829A_VACT_MASK) >> CMP_BT829A_VACT_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_vact_lo (board);

  return (tmp_val);
}

u_char 
cmp_get_hdly_lo (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_HDLY_LO_ADDRESS));
}

u_short
cmp_get_hdelay (u_char board) {
 
  u_short tmp_val;
 
  tmp_val = (u_short) cmp_get_crop (board);
  tmp_val = (tmp_val & CMP_BT829A_HDLY_MASK) >> CMP_BT829A_HDLY_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_hdly_lo (board);
 
  return (tmp_val);
}

u_char
cmp_get_hact_lo (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_HACT_LO_ADDRESS));
}

u_short
cmp_get_hactive (u_char board) {

  u_short tmp_val;

  tmp_val = (u_short) cmp_get_crop (board);
  tmp_val = (tmp_val & CMP_BT829A_HACT_MASK) >> CMP_BT829A_HACT_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_hact_lo (board);

  return (tmp_val);
}

u_char
cmp_get_hscal_hi (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_HSCAL_HI_ADDRESS));
}

u_char
cmp_get_hscal_lo (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_HSCAL_LO_ADDRESS));
}

u_short
cmp_get_hscale (u_char board) {
 
  u_short tmp_val;
 
  tmp_val = (u_short) cmp_get_hscal_hi (board);
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_hscal_lo (board);
 
  return (tmp_val);
}

u_char
cmp_get_bright (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_BRIGHT_ADDRESS));
}

u_char
cmp_get_control (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_CONTROL_ADDRESS));
}

u_char
cmp_get_con_lo (u_char board) { 
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_CON_LO_ADDRESS));
}

u_short
cmp_get_contrast (u_char board) {

  u_short tmp_val;

  tmp_val = (u_short) cmp_get_control (board);
  tmp_val = (tmp_val & CMP_BT829A_CON_MSB_MASK) >> CMP_BT829A_CON_MSB_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_con_lo (board);

  return (tmp_val);
}

u_char
cmp_get_sat_u_lo (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_SAT_U_LO_ADDRESS));
}

u_short
cmp_get_sat_u (u_char board) {
 
  u_short tmp_val;
 
  tmp_val = (u_short) cmp_get_control (board);
  tmp_val = (tmp_val & CMP_BT829A_SAT_U_MSB_MASK) >> CMP_BT829A_SAT_U_MSB_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_sat_u_lo (board);
 
  return (tmp_val);
}

u_char
cmp_get_sat_v_lo (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_SAT_V_LO_ADDRESS));
}

u_short
cmp_get_sat_v (u_char board) {
 
  u_short tmp_val;
 
  tmp_val = (u_short) cmp_get_control (board);
  tmp_val = (tmp_val & CMP_BT829A_SAT_V_MSB_MASK) >> CMP_BT829A_SAT_V_MSB_SHIFT;
  tmp_val = (tmp_val << 8) | (u_short) cmp_get_sat_v_lo (board);
 
  return (tmp_val);
}

u_char
cmp_get_hue (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_HUE_ADDRESS));
}

u_char
cmp_get_scloop (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_SCLOOP_ADDRESS));
}

u_char
cmp_get_oform (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_OFORM_ADDRESS));
}

u_char
cmp_get_vscal_hi (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_VSCAL_HI_ADDRESS));
}

u_char
cmp_get_vscal_lo (u_char board) {

  return (cmp_i2c_read (board,
                        CMP_BT829A_VSCAL_LO_ADDRESS));
}

u_short
cmp_get_vscale (u_char board) {

  int tmp_val;

  tmp_val = (int) cmp_get_vscal_hi (board);
  tmp_val = (tmp_val << 8) | (int) cmp_get_vscal_lo (board);
  tmp_val = ((0x10000 - tmp_val) & 0x1FFF) / 0x200 + 0x1;
				
  if (!(cmp_get_vscal_hi (board) & CMP_BT829A_INT_MASK)) {
    tmp_val *= 2;		/* Non-interlace Vscaling */
  }

  return ((u_short) tmp_val);
}

u_char
cmp_get_idcode (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_IDCODE_ADDRESS));
}

u_char
cmp_get_adly (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_ADLY_ADDRESS));
}

u_char
cmp_get_bdly (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_BDLY_ADDRESS));
}

u_char
cmp_get_adc (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_ADC_ADDRESS));
}
 
u_char
cmp_get_vtc (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_VTC_ADDRESS));
}
 
u_char
cmp_get_sreset (u_char board) {
 
  return (cmp_i2c_read (board,
                        CMP_BT829A_SRESET_ADDRESS));
}

int
cmp_set_status (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_STATUS_ADDRESS,
                         value));
}
 
int
cmp_set_iform (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_IFORM_ADDRESS,
                         value));
}

int
cmp_set_camera (u_char board, u_char value) {

  u_char tmp_val;

  tmp_val = cmp_get_iform (board);
  tmp_val = tmp_val & ~CMP_BT829A_MUXSEL_MASK;

  switch (value) {
  case 0:
    tmp_val = tmp_val | (CMP_CAMERA_0 << CMP_BT829A_MUXSEL_SHIFT);
    break;
  case 1:
    tmp_val = tmp_val | (CMP_CAMERA_1 << CMP_BT829A_MUXSEL_SHIFT);
    break;
  case 2:
    tmp_val = tmp_val | (CMP_CAMERA_2 << CMP_BT829A_MUXSEL_SHIFT);
    break;
  case 3:
    tmp_val = tmp_val | (CMP_CAMERA_3 << CMP_BT829A_MUXSEL_SHIFT);
    break;
  case 7:
    return (OK);
    break;
  default:
    icp.errno[board] = -EICP_NOSUPPORT;
    return (ERROR);
    break;
  }
  cmp_set_iform (board, tmp_val);

  return (OK);
}

int
cmp_set_tdec (u_char board, u_char value) {

  cmp_i2c_write (board,
                 CMP_BT829A_TDEC_ADDRESS,
                 0x00);
  return (cmp_i2c_write (board,
                         CMP_BT829A_TDEC_ADDRESS,
                         value));
}

int
cmp_set_crop (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_CROP_ADDRESS,
                         value));
}

int
cmp_set_vdly_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_VDLY_LO_ADDRESS,
                         value));
}

int
cmp_set_vdelay (u_char board, u_short value) {

  u_char tmp_val;

  tmp_val = cmp_get_crop (board) & ~CMP_BT829A_VDLY_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0300) >> 2));
  cmp_set_crop (board, tmp_val);

  tmp_val = (u_char) (value & 0x00FF);

  return (cmp_set_vdly_lo (board, tmp_val));
}

int
cmp_set_vact_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_VACT_LO_ADDRESS,
                         value));
}

int
cmp_set_vactive (u_char board, u_short value) {

  u_char tmp_val;

  tmp_val = cmp_get_crop (board) & ~CMP_BT829A_VACT_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0300) >> 4));
  cmp_set_crop (board, tmp_val);

  tmp_val = (u_char) (value & 0x00FF);

  return (cmp_set_vact_lo (board, tmp_val));
}

int
cmp_set_hdly_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_HDLY_LO_ADDRESS,
                         value));
}

int
cmp_set_hdelay (u_char board, u_short value) {

  u_char tmp_val;

  tmp_val = cmp_get_crop (board) & ~CMP_BT829A_HDLY_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0300) >> 6));
  cmp_set_crop (board, tmp_val);

  tmp_val = (u_char) (value & 0x00FF);

  return (cmp_set_hdly_lo (board, tmp_val));
}

int
cmp_set_hact_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_HACT_LO_ADDRESS,
                         value));
}

int
cmp_set_hactive (u_char board, u_short value) {

  u_char tmp_val;

  tmp_val = cmp_get_crop (board) & ~CMP_BT829A_HACT_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0300) >> 8));
  cmp_set_crop (board, tmp_val);

  tmp_val = (u_char) (value & 0x00FF);

  return (cmp_set_hact_lo (board, tmp_val));
}

int
cmp_set_hscal_hi (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_HSCAL_HI_ADDRESS,
                         value));
}

int
cmp_set_hscal_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_HSCAL_LO_ADDRESS,
                         value));
}

int
cmp_set_hscale (u_char board, u_short value) {
 
  u_char tmp_val;
 
  tmp_val = (u_char) ((value & 0xFF00) >> 8);
  cmp_set_hscal_hi (board, tmp_val);
  tmp_val = (u_char) (value & 0x00FF);
 
  return (cmp_set_hscal_lo (board, tmp_val));
}

int
cmp_set_bright (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_BRIGHT_ADDRESS,
                         value));
}

int
cmp_set_control (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_CONTROL_ADDRESS,
                         value));
}
 
int
cmp_set_con_lo (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_CON_LO_ADDRESS,
                         value));
}

int
cmp_set_contrast (u_char board, u_short value) {
 
  u_char tmp_val;
 
  tmp_val = cmp_get_control (board) & ~CMP_BT829A_CON_MSB_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0100) >> 6));
  cmp_set_control (board, tmp_val);
 
  tmp_val = (u_char) (value & 0x00FF);
 
  return (cmp_set_con_lo (board, tmp_val));
}
 
int
cmp_set_sat_u_lo (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_SAT_U_LO_ADDRESS,
                         value));
}

int
cmp_set_sat_u (u_char board, u_short value) {
 
  u_char tmp_val;
 
  tmp_val = cmp_get_control (board) & ~CMP_BT829A_SAT_U_MSB_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0100) >> 7));
  cmp_set_control (board, tmp_val);
 
  tmp_val = (u_char) (value & 0x00FF);
 
  return (cmp_set_sat_u_lo (board, tmp_val));
}

int
cmp_set_sat_v_lo (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_SAT_V_LO_ADDRESS,
                         value));
}

int
cmp_set_sat_v (u_char board, u_short value) {
 
  u_char tmp_val;
 
  tmp_val = cmp_get_control (board) & ~CMP_BT829A_SAT_V_MSB_MASK;
  tmp_val = tmp_val | ((u_char) ((value & 0x0100) >> 8));
  cmp_set_control (board, tmp_val);
 
  tmp_val = (u_char) (value & 0x00FF);
 
  return (cmp_set_sat_v_lo (board, tmp_val));
}

int
cmp_set_hue (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_HUE_ADDRESS,
                         value));
}
 
int
cmp_set_scloop (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_SCLOOP_ADDRESS,
                         value));
}
 
int
cmp_set_oform (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_OFORM_ADDRESS,
                         value));
}
 
int
cmp_set_vscal_hi (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_VSCAL_HI_ADDRESS,
                         value));
}

int
cmp_set_vscal_lo (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_VSCAL_LO_ADDRESS,
                         value));
}

int
cmp_set_vscale (u_char board, u_short value) {
 
  u_char tmp_val;
 
  tmp_val = cmp_get_vscal_hi (board) & ~CMP_BT829A_VSCAL_MASK;
  tmp_val = tmp_val | (u_char) ((value & 0x1F00) >> 8);
  cmp_set_vscal_hi (board, tmp_val);
  tmp_val = (u_char) (value & 0x00FF);
 
  return (cmp_set_vscal_lo (board, tmp_val));
}

int
cmp_set_adly (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_ADLY_ADDRESS,
                         value));
}
 
int
cmp_set_bdly (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_BDLY_ADDRESS,
                         value));
}
 
int
cmp_set_adc (u_char board, u_char value) {
 
  return (cmp_i2c_write (board,
                         CMP_BT829A_ADC_ADDRESS,
                         value));
}
 
int
cmp_set_vtc (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_VTC_ADDRESS,
                         value));
}

int
cmp_set_sreset (u_char board, u_char value) {

  return (cmp_i2c_write (board,
                         CMP_BT829A_SRESET_ADDRESS,
                         value));
}

/*
 * Special Initialization Handler for AM-STD Registers
 */

int
cmp_init_mxctl (u_char board) {

  return (cmp_write_mm (board,
                        CMP_MXCTL_OFFSET,
                        CMP_NO_MASK,
                        CMP_NO_MIN,
                        CMP_NO_MAX,
                        CMP_MXCTL_INIT));
}

int
cmp_init_scl (u_char board) {

  return (cmp_write_mm (board,
                        CMP_SCL_OFFSET,
                        CMP_NO_MASK,
                        CMP_NO_MIN,
                        CMP_NO_MAX,
                        CMP_SCL_INIT));
}

int
cmp_init_sda (u_char board) {

  return (cmp_write_mm (board,
                        CMP_SDA_OFFSET,
                        CMP_NO_MASK,
                        CMP_NO_MIN,
                        CMP_NO_MAX,
                        CMP_SDA_INIT));
}

int
cmp_init_nptrig (u_char board) {

  return (cmp_write_mm (board,
                        CMP_NPTRIG_OFFSET,
                        CMP_NO_MASK,
                        CMP_NO_MIN,
                        CMP_NO_MAX,
                        CMP_NPTRIG_INIT));
}

int
cmp_init_vdecregs (u_char board) {

  cmp_set_status   (board, CMP_BT829A_STATUS_INIT);
  cmp_set_iform    (board, CMP_BT829A_IFORM_INIT);
  cmp_set_tdec     (board, CMP_BT829A_TDEC_INIT);
  cmp_set_crop     (board, CMP_BT829A_CROP_INIT);
  cmp_set_vdly_lo  (board, CMP_BT829A_VDLY_LO_INIT);
  cmp_set_vact_lo  (board, CMP_BT829A_VACT_LO_INIT);
  cmp_set_hdly_lo  (board, CMP_BT829A_HDLY_LO_INIT);
  cmp_set_hact_lo  (board, CMP_BT829A_HACT_LO_INIT);
  cmp_set_hscal_hi (board, CMP_BT829A_HSCAL_HI_INIT);
  cmp_set_hscal_lo (board, CMP_BT829A_HSCAL_LO_INIT);
  cmp_set_bright   (board, CMP_BT829A_BRIGHT_INIT);
  cmp_set_control  (board, CMP_BT829A_CONTROL_INIT);
  cmp_set_con_lo   (board, CMP_BT829A_CON_LO_INIT);
  cmp_set_sat_u_lo (board, CMP_BT829A_SAT_U_LO_INIT);
  cmp_set_sat_v_lo (board, CMP_BT829A_SAT_V_LO_INIT);
  cmp_set_hue      (board, CMP_BT829A_HUE_INIT);
  cmp_set_scloop   (board, CMP_BT829A_SCLOOP_INIT);
  cmp_set_oform    (board, CMP_BT829A_OFORM_INIT);
  cmp_set_vscal_hi (board, CMP_BT829A_VSCAL_HI_INIT);
  cmp_set_vscal_lo (board, CMP_BT829A_VSCAL_LO_INIT);
  cmp_set_adly     (board, CMP_BT829A_ADLY_INIT);
  cmp_set_bdly     (board, CMP_BT829A_BDLY_INIT);
  cmp_set_adc      (board, CMP_BT829A_ADC_INIT);
  cmp_set_vtc      (board, CMP_BT829A_VTC_INIT);

  return (OK);
}

/* 
 * Miscellanous functions
 */

#ifdef DEBUG_ICPCI
/* make register dump */
void
cmp_dump_regs (u_char board) {

  u_char count;

  for (count=0; count < CMP_REG_SIZE; count++) {
    icp_printf ("icpci[%d]: %s %#04x\n", board, cmp_reg[count],
                cmp_read_mm (board, count, CMP_NO_MASK,
                CMP_NO_MIN, CMP_NO_MAX));
  }

  for (count=0; count < CMP_BT829A_REG_SIZE; count++) {
    icp_printf ("icpci[%d]: bt829a : %s %#04x\n",
                board, cmp_bt829a_reg[count], cmp_i2c_read (board, count));
  }
}
#endif
