/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: amvsReg.h,v 5.1 1999/05/04 12:37:27 matthias Exp $
 *
 */

#ifndef _AMVSREG_H
#define _AMVSREG_H

#if defined(linux) && defined(__KERNEL__) || defined(__CONFIG__)

/* AM-VS Hardware Reference Manual; Imaging Technology Inc. */
/* Document Number 47-H44004-02; March 1993 */

/* R-O   Read-Only Register */
/* W-O   Write-Only Register */
/* R/W   Read/Write Register */
/* R/W1C Clear on Write One to Register */
/* R/W0C Clear on Write Zero to Register */

/* AM-VS Register Set (16bit access only), pages 3-1 to 3-25 */
/* ********************************************************* */

/* 
 * Prototypes
 */

/* read handler */

u_char vs_get_rev (u_char);
u_char vs_get_tgid (u_char);
u_char vs_get_tsel (u_char);
u_char vs_get_clksrc (u_char);
u_char vs_get_lpfsel (u_char);
u_char vs_get_iscale (u_char);
u_char vs_get_smode (u_char);
u_char vs_get_camsel (u_char);
u_char vs_get_tsrc (u_char);
u_char vs_get_lenpol (u_char);
u_char vs_get_fldpol (u_char);
u_char vs_get_fenpol (u_char);
u_char vs_get_pclkpol (u_char);
u_char vs_get_eclken (u_char);
u_char vs_get_vbstat (u_char);
u_char vs_get_fldsten (u_char);
u_char vs_get_vsten (u_char);
u_char vs_get_trigen (u_char);
u_char vs_get_trigsrc (u_char);
u_char vs_get_fmode  (u_char);
u_char vs_get_tmode (u_char);
u_char vs_get_trigcy (u_char);
u_char vs_get_strbdly (u_char);
u_short vs_get_hoff (u_char);
u_short vs_get_hsz (u_char);
u_short vs_get_voff (u_char);
u_short vs_get_vsz (u_char);
u_short vs_get_froff (u_char);
u_char vs_get_cppos (u_char);
u_char vs_get_cpen (u_char);
u_char vs_get_cpsrc (u_char);
u_char vs_get_strbsel (u_char);
u_char vs_get_strbpol (u_char);
u_char vs_get_cpst (u_char);
u_char vs_get_lutdata (u_char, struct vs_lut_t*); 
u_short vs_get_nref (u_char);
u_short vs_get_pref (u_char);

/* write handler */

int vs_set_tsel (u_char, u_char);
int vs_set_clksrc (u_char, u_char);
int vs_set_lpfsel (u_char, u_char);
int vs_set_iscale (u_char, u_char);
int vs_set_smode (u_char, u_char);
int vs_set_camsel (u_char, u_char);
int vs_set_tsrc (u_char, u_char);
int vs_set_lenpol (u_char, u_char);
int vs_set_fldpol (u_char, u_char);
int vs_set_fenpol (u_char, u_char);
int vs_set_pclkpol (u_char, u_char);
int vs_set_eclken (u_char, u_char);
int vs_set_fldsten (u_char, u_char);
int vs_set_vsten (u_char, u_char);
int vs_set_trigen (u_char, u_char);
int vs_set_trigsrc (u_char, u_char);
int vs_set_fmode (u_char, u_char);
int vs_set_tmode (u_char, u_char);
int vs_set_strbdly (u_char, u_char);
int vs_set_hoff (u_char, u_short);
int vs_set_hsz (u_char, u_short);
int vs_set_voff (u_char, u_short);
int vs_set_vsz (u_char, u_short);
int vs_set_froff (u_char, u_short);
int vs_set_cppos (u_char, u_char);
int vs_set_cpen (u_char, u_char);
int vs_set_cpsrc (u_char, u_char);
int vs_set_strbsel (u_char, u_char);
int vs_set_strbpol (u_char, u_char);
int vs_set_cpst (u_char, u_char);
int vs_set_lutdata (u_char, struct vs_lut_t*);
int vs_set_nref (u_char, u_short);
int vs_set_pref (u_char, u_short);

/* init handler */

int vs_init_con1 (u_char);
int vs_init_con2 (u_char);
int vs_init_trigcon (u_char);
int vs_init_cpcon (u_char);
int vs_init_adccon (u_char);

/* misc handler */

int vs_sw_trig (u_char);
int vs_lin_lut (u_char);
void vs_dump_regs (u_char);

/* end of prototypes */

#define VS_DISABLE		0
#define VS_ENABLE		1

#define VS_REG_SIZE		19	/* size of register set */

#define VS_NO_MASK		0xFFFF	/* no checking */
#define VS_NO_MIN		0X0000
#define VS_NO_MAX		0xFFFF

#define VS_RES_INIT		0x0000	/* reset reserved bits to zero */

#define VS_MAGIC_MASK		0xC7FF	/* TGID always visible */

/*    u_short	  modid		Module and Timing ID */
/* ------------------------------------------------- */

#define VS_MODID_OFFSET		0x00

/*    u_short      rev:3         Revision R-O */

#define VS_REV_MASK		0x0007
#define VS_REV_SHIFT		0

/*    u_short      type:5        Board Type R-O */

/*    unified access interface for every module type, see icpciAux.c */

/*    u_short      res_1:5       reserved - reset to 0 */

/*    u_short      tgid:3        Timing Generator Type R-O */

#define VS_TGID_MASK		0xE000
#define VS_TGID_SHIFT		13

/*    u_short      con1		Control1 */
/* ------------------------------------- */

#define VS_CON1_OFFSET		0x01
#define VS_CON1_INIT		0x015F
				/* CCIR 736x574 (14.18 Mhz) */
				/* PLL- Mode */
				/* 4.3 MHz -3dB cut off */
				/* Scale by 1.5 */
				/* Interlaced */
				/* Camera 0 */

/*    u_short      tsel:3	Timing Select R/W */

#define VS_TSEL_MASK		0x0007
#define VS_TSEL_SHIFT		0

/* Timing Generator Type 0 (Standard) */

#define VS_T_0_10_00_RS170	0	/* 512x484 RS170 */
#define VS_T_0_12_28_RS170	2	/* 640x484 RS170 */
#define VS_T_0_14_32_RS170	3	/* 756x484 RS170 */
#define VS_T_0_10_00_CCIR	4	/* 512x574 CCIR */
#define VS_T_0_15_00_CCIR	6	/* 768x574 CCIR */
#define VS_T_0_14_18_CCIR	7	/* 736x574 CCIR */

/* Timing Generator Type 1 (MC1000P) */

#define VS_T_1_10_00_RS170	0	/* 512x484 RS170 */
#define VS_T_1_20_00_MC1000P	1	/* 1000x504 MC1000P */ 
#define VS_T_1_12_28_RS170	2	/* 640x484 RS170 */
#define VS_T_1_14_32_RS170	3	/* 756x484 RS170 */
#define VS_T_1_10_00_CCIR	4	/* 512x574 CCIR */
#define VS_T_1_15_00_CCIR	6	/* 768x574 CCIR */
#define VS_T_1_14_18_CCIR	7	/* 736x574 CCIR */

/* Timing Generator Type 2 (Dage 81) */

#define VS_T_2_10_00_RS170	0	/* 512x484 RS170 */
#define VS_T_2_07_50_DAGE81	1	/* 1024x1050 Dage 81 2.5fps */
#define VS_T_2_12_28_RS170	2	/* 640x484 RS170 */
#define VS_T_2_14_32_RS170	3	/* 756x484 RS170 */
#define VS_T_2_10_00_CCIR	4	/* 512x574 CCIR */
#define VS_T_2_14_66_DAGE81	5	/* 1024x1050 Dage 81 6.25fps */
#define VS_T_2_15_00_CCIR	6	/* 768x574 CCIR */
#define VS_T_2_14_18_CCIR	7	/* 736x574 CCIR */

/*    u_short      clksrc:1	Clock Source Select R/W */

#define VS_CLKSRC_MASK		0x0008
#define VS_CLKSRC_SHIFT		3
#define VS_CLKSRC_XTAL		0	/* Crystal Mode */
#define VS_CLKSRC_PLL		1	/* Phase - Locked Loop Mode */

/*    u_short      lpfsel:2      Low - Pass Filter Select R/W */ 

#define VS_LPFSEL_MASK		0x0030
#define VS_LPFSEL_SHIFT		4
#define VS_LPF_3_0		0	/*  3.0 MHz - 3dB bandwidth */
#define VS_LPF_4_3		1	/*  4.3 MHz - 3dB bandwidth */
#define VS_LPF_6_5		2	/*  6.5 MHz - 3dB bandwidth */
#define VS_LPF_12_0		3	/* 12.0 MHz - 3dB bandwidth */

/*    u_short      iscale:2	Video Input Scale Factor R/W */

#define VS_ISCALE_MASK		0x00C0
#define VS_ISCALE_SHIFT		6
#define VS_SCALE_1_0		0	/* Scale Video by 1.0 */
#define VS_SCALE_1_5		1	/* Scale Video by 1.5 */
#define VS_SCALE_0_5		2	/* Scale Video by 0.5 */
#define VS_SCALE_0_75		3	/* Scale Video by 0.75 */

/*    u_short      smode:1	Scan Mode R/W */

#define VS_SMODE_MASK		0x0100
#define VS_SMODE_SHIFT		8
#define VS_NON_ILACE		0	/* Non - Interlaced Camera */
#define VS_ILACE		1	/* Interlaced Camera */

/*    u_short      camsel:2	Camera Select R/W */

#define VS_CAMSEL_MASK		0x0600
#define VS_CAMSEL_SHIFT		9
#define VS_CAM_0		0	/* Camera 0 enabled */
#define VS_CAM_1		1	/* Camera 1 enabled */
#define VS_CAM_2		2	/* Camera 2 enabled */
#define VS_CAM_3		3	/* Camera 3 enabled */

/*    u_short      res_1:5	reserved - reset to 0 */

/*    u_short      con2		Control2 */
/* ------------------------------------- */

#define VS_CON2_OFFSET		0x02
#define VS_CON2_INIT		0x031C
				/* Standatd Timing Generator drives ADC */
				/* LEN High no invert */
				/* FLD no invert */
				/* FEN High no invert */
				/* Sample PCLK on Falling Edge */
				/* External Clock Low */
				/* Field Sync Strip Enable */
				/* Vertical Sync Strip Enable */

/*    u_short      tsrc:2	Timing Source Select R/W */

#define VS_TSRC_MASK		0x0003
#define VS_TSRC_SHIFT		0
#define VS_TSRC_STD		0	/* Standard Timing Generator */
#define VS_TSRC_EXT0		1	/* External Timing from Camera 0 */
#define VS_TSRC_EXT1		2	/* External Timing from Camera 1 */
#define VS_TSRC_DIF		3	/* External Differential Timing */

/*    u_short      lenpol:1	Line Enable Polarity R/W */

#define VS_LENPOL_MASK		0x0004
#define VS_LENPOL_SHIFT		2
#define VS_LEN_LO		0	/* LEN Input active low, invert */
#define VS_LEN_HI		1	/* LEN Input active high, no invert */

/*    u_short      fldpol:1	Field Polarity R/W */

#define VS_FLDPOL_MASK		0x0008
#define VS_FLDPOL_SHIFT		3
#define VS_FLD_INV		0	/* low = odd, high = even (invert) */
#define VS_FLD_NO_INV		1	/* low = even, high = odd (no invert) */

/*    u_short      fenpol:1	Frame Enable Polarity R/W */

#define VS_FENPOL_MASK		0x0010
#define VS_FENPOL_SHIFT		4
#define VS_FEN_LO		0	/* FEN Input active low, invert */
#define VS_FEN_HI		1	/* FEN Input active high, no invert */

/*    u_short      pclkpol:1	Pixel Clock Polarity R/W */

#define VS_PCLKPOL_MASK		0x0020
#define VS_PCLKPOL_SHIFT	5	
#define VS_PCLK_FA		0	/* Sample on falling edge (invert) */
#define VS_PCLK_RI		1	/* Sample on rising edge (no invert) */

/*    u_short      eclken:1	External Clock Enable R/W */

#define VS_ECLKEN_MASK		0x0040
#define VS_ECLKEN_SHIFT		6

/*    u_short      vbstat:1	Vertical Blank Status R-O */

#define VS_VBSTAT_MASK		0x0020
#define VS_VBSTAT_SHIFT		5
#define VS_IN_VB		0	/* IN camera vertical blank */
#define VS_OUT_VB		1	/* OUT of camera vertical blank */

/*    u_short      fldsten:1	Field Stripper Enable R/W */

#define VS_FLDSTEN_MASK		0x0100
#define VS_FLDSTEN_SHIFT	8

/*    u_short	  vsten:1	Vertical Stripper Enable R/W */

#define VS_VSTEN_MASK		0x0200
#define VS_VSTEN_SHIFT		9

/*    u_short      res_1:6	reserved - reset to 0 */

/*    u_short      trigcon	Trigger Control */
/* -------------------------------------------- */

#define VS_TRIGCON_OFFSET	0x03
#define VS_TRIGCON_INIT		0x0008
				/* Trigger disable*/
				/* External Trigger */
				/* Stays Low One Line */
				/* Trigger on Frame - Slow Strobe */
				/* Strobe Delay = 0 */

/*    u_short      trigen:1	Trigger Enable R/W */

#define VS_TRIGEN_MASK		0x0001
#define VS_TRIGEN_SHIFT		0

/*    u_short      trigsrc:1	Trigger Source R/W */

#define VS_TRIGSRC_MASK		0x0002
#define VS_TRIGSRC_SHIFT	1
#define VS_TRIG_EXT		0	/* External Trigger */
#define VS_TRIG_SOFT		1	/* Software Trigger */

/*    u_short      res_1:1	reserved - reset to 0 */

/*    u_short      fmode:1	Frame Reset Mode R/W */

#define VS_FMODE_MASK		0x0008
#define VS_FMODE_SHIFT		3
#define VS_FMODE_LOW_VB		0	/* reset pulse stays low one VB */
#define VS_FMODE_LOW_LINE	1	/* reset pulse stays low one Line */

/*    u_short      tmode:2	Trigger Mode R/W */

#define VS_TMODE_MASK		0x0030
#define VS_TMODE_SHIFT		4
#define VS_TMODE_SLOW_STROBE	0	/* Trigger on Frame - slow strobe */
#define VS_TMODE_FRAME_RESET	1	/* Frame Reset */
#define VS_TMODE_LINE_SCAN	2	/* Line Scan */
#define VS_TMODE_FAST_STROBE	3	/* Trigger on Frame - fast strobe */

/*    u_short      trigcy:1	Trigger Cycle Status R-O */

#define VS_TRIGCY_MASK		0x0040
#define VS_TRIGCY_SHIFT		6
#define VS_IN_TRIGCY		0	/* Trigger Cycle in Progress */
#define VS_OUT_TRIGCY		1	/* No Trigger Cycle in Progress */

/*    u_short      strbdly:4	Strobe Delay R/W */

#define VS_STRBDLY_MASK		0x0780
#define VS_STRBDLY_SHIFT	7	
#define VS_STRBDLY_MIN		VS_NO_MIN
#define VS_STRBDLY_MAX		VS_STRBDLY_MASK >> VS_STRBDLY_SHIFT

/*    u_short      res_2:5	reserved - reset to 0 */

/*    u_short      hoff		Horizontal Offset R/W */
/*                              0:8 -> 0-511 | hoff+4*(hsz+1) < pixel/line - 7 */

#define VS_HOFF_OFFSET		0x04
#define VS_HOFF_MASK		0x01FF
#define VS_HOFF_MIN		VS_NO_MIN
#define VS_HOFF_MAX		VS_HOFF_MASK
#define VS_HOFF_INIT		148	/* Sony XC75-CE */

/*    u_short      hsz		Horizontal Size R/W */
/*                              0:9 -> size = 4 * ( hsz + 1 ) */

#define VS_HSZ_OFFSET		0x05
#define VS_HSZ_MASK		0x03FF
#define VS_HSZ_MIN		VS_NO_MIN
#define VS_HSZ_MAX		VS_HSZ_MASK
#define VS_HSZ_INIT		736	/* Sony XC75-CE */

/*    u_short      voff		Vertical Offset R/W */

#define VS_VOFF_OFFSET		0x06
#define VS_VOFF_MASK		0x03FF
#define VS_VOFF_MIN		VS_NO_MIN
#define VS_VOFF_MAX		VS_VOFF_MASK
#define VS_VOFF_INIT		22	/* Sony XC75-CE */

/*    u_short      vsz		Vertical Size R/W */

#define VS_VSZ_OFFSET		0x07
#define VS_VSZ_MASK		0x03FF
#define VS_VSZ_MIN		VS_NO_MIN
#define VS_VSZ_MAX		VS_VSZ_MASK
#define VS_VSZ_INIT		286	/* Sony XC75-CE (572) */

/*    u_short      froff		Frame Reset Offset R/W */

#define VS_FROFF_OFFSET		0x08
#define VS_FROFF_MASK		0x01FF
#define VS_FROFF_MIN		VS_NO_MIN
#define VS_FROFF_MAX		VS_FROFF_MASK
#define VS_FROFF_INIT		0x14

/*    u_short      cpcon		Clamp Control */
/* ------------------------------------------ */

#define VS_CPCON_OFFSET		0x09
#define VS_CPCON_INIT		0x0040
				/* Clamp Posistion = 0 */ 
				/* Clamp Enable */ 
				/* Automatic Clamp Pulse */ 
				/* Strobe Output 2 */ 
				/* Strobe Low */ 

/*    u_short      cppos:6	Clamp Position R/W */

#define VS_CPPOS_MASK		0x003F
#define VS_CPPOS_SHIFT		0
#define VS_CPPOS_MIN		VS_NO_MIN
#define VS_CPPOS_MAX		VS_CPPOS_MASK >> VS_CPPOS_SHIFT

/*    u_short      cpen:1	Clamp Enable R/W */

#define VS_CPEN_MASK		0x0040
#define VS_CPEN_SHIFT		6

/*    u_short      cpsrc:1	Clamp Source R/W */

#define VS_CPSRC_MASK		0x0080
#define VS_CPSRC_SHIFT		7
#define VS_CPSRC_AUTO		0	/* Automatic Clamp Pulse Enabled */
#define VS_CPSRC_PROG		1	/* Programmable Clamp Pulse Enabled */

/*    u_short      strbsel:1     Strobe Select R/W */

#define VS_STRBSEL_MASK		0x0100
#define VS_STRBSEL_SHIFT	8
#define VS_STRB_1		0	/* Strobe 1 Output Enabled */
#define VS_STRB_2		1	/* Strobe 2 Output Enabled */

/*    u_short      strbpol:1     Strobe Polarity R/W */

#define VS_STRBPOL_MASK		0x0200
#define VS_STRBPOL_SHIFT	9
#define VS_STRB_LO		0	/* Strobe Active Low Pulse */
#define VS_STRB_HI		1	/* Strobe Active High Pulse */

/*    u_short      res_1:2	reserved - reset to 0 */

/*    u_short      cpst:1        Clamp Counter Start R/W */

#define VS_CPST_MASK		0x1000
#define VS_CPST_SHIFT		12
#define VS_CPST_FA		0	/* Clamp Counter enabled by falling */
					/* edge of LEN (LENPOL = 1) */
#define VS_CPST_RI		1	/* Clamp Counter enabled by rising */
					/* edge of LEN (LENPOL = 1) */

/*    u_short      res_2:3	reserved - reset to 0 */

/*    u_short      swtrig	Software Trigger W-O */

#define VS_SWTRIG_OFFSET	0x0A
#define VS_SWTRIG_MASK		0x00FF
#define VS_SWTRIG_MIN		VS_NO_MIN
#define VS_SWTRIG_MAX		VS_SWTRIG_MASK		
#define VS_SWTRIG		0x00FF	/* Trigger per Software */

/*    u_short      res_1[4]	reserved - reset to 0 */

/*    u_short      lutadr	Input Lookup Table Address R/W */

#define VS_LUTADDR_OFFSET	0x10
#define VS_LUTADDR_MASK		0x00FF
#define VS_LUTADDR_MIN		VS_NO_MIN
#define VS_LUTADDR_MAX		VS_LUTADDR_MASK
#define VS_LUTADDR_INIT		0x0000		

/* SECOND FUNCTION: ADDRESSES ALSO THE REGISTERS OF THE ADC CHIP */

#define VS_LUTADDR_ADC_MASK	0x0003
#define VS_LUTADDR_ADC_SHIFT	0
#define	VS_LUTADDR_ADC_CR	0	/* write ADC Command Register */
#define VS_LUTADDR_ADC_NR	1	/* write ADC Negative Ref. Register */
#define VS_LUTADDR_ADC_PR	2	/* write ADC Positive Ref. Register */

/*    u_short      lutdata       Input LUT Data R/W */

#define VS_LUTDATA_OFFSET	0x11
#define VS_LUTDATA_MASK		0x00FF
#define VS_LUTDATA_MIN		VS_NO_MIN
#define VS_LUTDATA_MAX		VS_LUTDATA_MASK

/*    u_short      adccon	ADC Control R/W */

#define VS_ADCCON_OFFSET	0x12
#define VS_ADCCON_MASK		0xFFFF
#define VS_ADCCON_INIT		0x0000

/*    u_short      nref		ADC Negative Refenrence R/W */

#define VS_NREF_OFFSET		VS_ADCCON_OFFSET
#define VS_NREF_MASK		0x00FF
#define VS_NREF_MIN		VS_NO_MIN
#define VS_NREF_MAX		VS_NREF_MASK
#define VS_NREF_INIT		0	/* CCIR / RS170 Camera */ 

/*    u_short      pref		ADC Positive Refenrence R/W */

#define VS_PREF_OFFSET		VS_ADCCON_OFFSET
#define VS_PREF_MASK		0x00FF
#define VS_PREF_MIN		VS_NO_MIN
#define VS_PREF_MAX		VS_PREF_MASK
#define VS_PREF_INIT		141	/* CCIR / RS170 Camera */ 

#endif /* linux && __KERNEL__ || __CONFIG__ */

#endif /* _AMVSREG_H */

