/*		 ICP-PCI Kernel module free public release
 *
 *		       Copyright (C) 1996-99 GOM mbH.
 *	         Rebenring 33, 38106 Braunschweig, GERMANY
 *		            All rights reserved
 *
 * $Id: config.y,v 5.1 1999/05/04 12:37:27 matthias Exp $
 *
 */

%{

#include	<stdio.h>
#include	<fcntl.h>
#include	<strings.h>

#include	"icpciExt.h"
#include	"amvsReg.h"
#include	"amdigReg.h"
#include	"ampcvReg.h"
#include	"config.h"

int		count = 0;

int		i_valid = FALSE,	/* identifier found */
		p_valid = FALSE;	/* parameter found */

int		board = -1,
		module = -1,
		camera = -1;

int		presync_i = FALSE,
		presync_ii = FALSE,
		y_valid = FALSE,
		boardfound = FALSE,
		modulefound = FALSE,
		camerafound = FALSE,	
		readout = FALSE;

char            camera_name[MAX_ENTRY_LENGTH];

union icp_dev_t	*camcnf;

const struct icp_cnf_flag_t	*icp_cnf_flag;
const struct icp_cnf_enum_t	*icp_cnf_enum;
const struct icp_cnf_num_t	*icp_cnf_num;

#ifdef DEBUG_ICPCI
const struct icp_cnf_flag_debug_t	*icp_cnf_flag_debug;
const struct icp_cnf_enum_debug_t	*icp_cnf_enum_debug;
const struct icp_cnf_num_debug_t	*icp_cnf_num_debug;
#endif

extern FILE	*yyin;

%}

%union {
  	long long_val;
       	char char_val[64];
       };


%token BOARD_0 BOARD_1 BOARD_2 BOARD_3
%token MODULE_FA MODULE_VS MODULE_C1 MODULE_DG MODULE_PV
%token CAMERA_0 CAMERA_1 CAMERA_2 CAMERA_3
%token PARAM_FOLLOWS
%token <char_val> WORD
%token <long_val> DIGIT
%token <char_val> STRING

%%

board:
	|	board_entry board
	;

board_entry:	board_begin parameter_list module
		{ presync_i = FALSE; }
	;

board_begin:	BOARD_0
		{ if (board == 0) {
		    presync_i = boardfound = TRUE;
		  }
		}
	|	BOARD_1
		{ if (board == 1) {
		    presync_i = boardfound = TRUE;
		  }
		}
	|	BOARD_2
		{ if (board == 2) {
		    presync_i = boardfound = TRUE;
		  }
		}
	|	BOARD_3
		{ if (board == 3) {
		    presync_i = boardfound = TRUE;
		  }
		}
	;

module:		module_begin parameter_list camera
		{ presync_ii = FALSE; }
	;

module_begin:	MODULE_FA
		{ if (presync_i && module == ICP_AMFA) {
		    presync_ii = modulefound = TRUE;
		  }
		}
	|	MODULE_C1
		{ if (presync_i && module == ICP_AMC1) {
		    presync_ii = modulefound = TRUE;
		  }
		}
	|	MODULE_VS
		{ if (presync_i && module == ICP_AMVS) {
		    icp_cnf_flag = vs_cnf_flag;
		    icp_cnf_enum = vs_cnf_enum;
		    icp_cnf_num = vs_cnf_num;
#ifdef DEBUG_ICPCI
		    icp_cnf_flag_debug = vs_cnf_flag_debug;
		    icp_cnf_enum_debug = vs_cnf_enum_debug;
		    icp_cnf_num_debug = vs_cnf_num_debug;
#endif
		    presync_ii = modulefound = TRUE;
		  }
		}
	|	MODULE_DG
		{ if (presync_i && module == ICP_AMDG && camera == 0) {
		    icp_cnf_flag = dg_cnf_flag;
		    icp_cnf_enum = dg_cnf_enum;
		    icp_cnf_num = dg_cnf_num;
#ifdef DEBUG_ICPCI
		    icp_cnf_flag_debug = dg_cnf_flag_debug;
		    icp_cnf_enum_debug = dg_cnf_enum_debug;
		    icp_cnf_num_debug = dg_cnf_num_debug;
#endif
		    presync_ii = modulefound = TRUE;
		  }
		}
	|	MODULE_PV
		{ if (presync_i && module == ICP_AMPV) {
		    icp_cnf_flag = pv_cnf_flag;
		    icp_cnf_enum = pv_cnf_enum;
		    icp_cnf_num = pv_cnf_num;
#ifdef DEBUG_ICPCI
		    icp_cnf_flag_debug = pv_cnf_flag_debug;
		    icp_cnf_enum_debug = pv_cnf_enum_debug;
		    icp_cnf_num_debug = pv_cnf_num_debug;
#endif
		    presync_ii = modulefound = TRUE;
		  }
		}
	;

camera:
	|	camera_entry camera
	;

camera_entry:	camera_begin parameter_list
		{ y_valid = FALSE; }
	;

camera_begin:	CAMERA_0
		{ if (presync_ii && camera == 0) {
		    readout = y_valid = camerafound = TRUE;
		  }
		}
	|	CAMERA_1
		{ if (presync_ii && camera == 1) {
		    readout = y_valid = camerafound = TRUE;
		  }
		}
	|	CAMERA_2
		{ if (presync_ii && camera == 2) {
		    readout = y_valid = camerafound = TRUE;
		  }
		}
	|	CAMERA_3
		{ if (presync_ii && camera == 3) {
		    readout = y_valid = camerafound = TRUE;
		  }
		}
	;

parameter_list:
	|	parameter parameter_list
	;

parameter:	WORD
		{ if (y_valid) {
		    while (*icp_cnf_flag[count].identifier != '\0') {
		      if (!strcmp (icp_cnf_flag[count].identifier,$1)) {
		        ICP_PRINTF ("\t\t%s: %s\n",
				    icp_cnf_flag_debug[count].reg_name,
				    icp_cnf_flag_debug[count].meaning);
			i_valid = TRUE;
			switch (module) {
			case ICP_AMFA:
			  break;
			case ICP_AMC1:
			  break;
			case ICP_AMVS:
			  switch (count) {
			  case 0:
			  case 1:
			    camcnf->vs_cam.clksrc = icp_cnf_flag[count].value;
			    break;
			  case 2:
			  case 3:
			    camcnf->vs_cam.smode = icp_cnf_flag[count].value;
			    break;
			  case 4:
			  case 5:
			    camcnf->vs_cam.lenpol = icp_cnf_flag[count].value;
			    break;
			  case 6:
			  case 7:
			    camcnf->vs_cam.fldpol = icp_cnf_flag[count].value;
			    break;
			  case 8:
			  case 9:
			    camcnf->vs_cam.fenpol = icp_cnf_flag[count].value;
			    break;
			  case 10:
			  case 11:
			    camcnf->vs_cam.pclkpol = icp_cnf_flag[count].value;
			    break;
			  case 12:
			  case 13:
			    camcnf->vs_cam.eclken = icp_cnf_flag[count].value;
			    break;
			  case 14:
			  case 15:
			    camcnf->vs_cam.fldsten = icp_cnf_flag[count].value;
			    break;
			  case 16:
			  case 17:
			    camcnf->vs_cam.vsten = icp_cnf_flag[count].value;
			    break;
			  case 18:
			  case 19:
			    camcnf->vs_cam.trgen = icp_cnf_flag[count].value;
			    break;
			  case 20:
			  case 21:
			    camcnf->vs_cam.cpen = icp_cnf_flag[count].value;
			    break;
			  case 22:
			  case 23:
			    camcnf->vs_cam.strbpol = icp_cnf_flag[count].value;
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			case ICP_AMDG:
			  switch (count) {
			  case 0:
			  case 1:
			  case 2:
			  case 3:
			  case 4:
			  case 5:
			  case 6:
			  case 7:
			  case 8:
			  case 9:
			    camcnf->dg_cam.fsel = icp_cnf_flag[count].value;
			    break;
			  case 10:
			  case 11:
			  case 12:
			  case 13:
			  case 14:
			  case 15:
			  case 16:
			  case 17:
			    camcnf->dg_cam.clkdiv = icp_cnf_flag[count].value;
			    break;
			  case 18:
			  case 19:
			    camcnf->dg_cam.xillen = icp_cnf_flag[count].value;
			    break;
			  case 20:
			  case 21:
			    camcnf->dg_cam.pclkpol = icp_cnf_flag[count].value;
			    break;
			  case 22:
			  case 23:
			    camcnf->dg_cam.lenpol = icp_cnf_flag[count].value;
			    break;
			  case 24:
			  case 25:
			    camcnf->dg_cam.fenpol = icp_cnf_flag[count].value;
			    break;
			  case 26:
			  case 27:
			    camcnf->dg_cam.fldpol = icp_cnf_flag[count].value;
			    break;
			  case 28:
			  case 29:
			    camcnf->dg_cam.smode = icp_cnf_flag[count].value;
			    break;
			  case 30:
			  case 31:
			    camcnf->dg_cam.ilmode = icp_cnf_flag[count].value;
			    break;
			  case 32:
			  case 33:
			    camcnf->dg_cam.lensm = icp_cnf_flag[count].value;
			    break;
			  case 34:
			  case 35:
			    camcnf->dg_cam.nolmiss = icp_cnf_flag[count].value;
			    break;
			  case 36:
			  case 37:
			    camcnf->dg_cam.lvar = icp_cnf_flag[count].value;
			    break;
			  case 38:
			  case 39:
			  case 40:
			  case 41:
			  case 42:
			  case 43:
			    camcnf->dg_cam.exclk = icp_cnf_flag[count].value;
			    break;
			  case 44:
			  case 45:
			    camcnf->dg_cam.exsen = icp_cnf_flag[count].value;
			    break;
			  case 46:
			  case 47:
			    camcnf->dg_cam.expol = icp_cnf_flag[count].value;
			    break;
			  case 48:
			  case 49:
			    camcnf->dg_cam.prien = icp_cnf_flag[count].value;
			    break;
			  case 50:
			  case 51:
			    camcnf->dg_cam.prip = icp_cnf_flag[count].value;
			    break;
			  case 52:
			  case 53:
			  case 54:
			  case 55:
			    camcnf->dg_cam.psize = icp_cnf_flag[count].value;
			    break;
			  case 56:
			  case 57:
			    camcnf->dg_cam.ilpixel = icp_cnf_flag[count].value;
			    break;
			  case 58:
			  case 59:
			    camcnf->dg_cam.trigen = icp_cnf_flag[count].value;
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			case ICP_AMPV:
			  switch (count) {
			  case 0:
			  case 1:
			    camcnf->pv_cam.vsyncpw.b.hsyncpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 2:
			  case 3:
			    camcnf->pv_cam.vsyncpw.b.vsyncpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 4:
			  case 5:
			    camcnf->pv_cam.strbdly.b.lenpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 6:
			  case 7:
			    camcnf->pv_cam.strbdly.b.fenpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 8:
			  case 9:
			    camcnf->pv_cam.frstdly.b.frstonv =
			      icp_cnf_flag[count].value;
			    break;
			  case 10:
			  case 11:
			    camcnf->pv_cam.frstdly.b.frstmd =
			      icp_cnf_flag[count].value;
			    break;
			  case 12:
			  case 13:
			    camcnf->pv_cam.frstdly.b.frstpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 14:
			  case 15:
			    camcnf->pv_cam.frstdly.b.frstsz =
			      icp_cnf_flag[count].value;
			    break;
			  case 16:
			  case 17:
			    camcnf->pv_cam.con1.b.syncsel =
			      icp_cnf_flag[count].value;
			    break;
			  case 18:
			  case 19:
			    camcnf->pv_cam.con1.b.chsyncen =
			      icp_cnf_flag[count].value;
			    break;
			  case 20:
			  case 21:
			    camcnf->pv_cam.con1.b.vsyncen =
			      icp_cnf_flag[count].value;
			    break;
			  case 22:
			  case 23:
			    camcnf->pv_cam.con1.b.clksrc =
			      icp_cnf_flag[count].value;
			    break;
			  case 24:
			  case 25:
			    camcnf->pv_cam.con1.b.lpfsel =
			      icp_cnf_flag[count].value;
			    break;
			  case 26:
			  case 27:
			    camcnf->pv_cam.con1.b.vscan =
			      icp_cnf_flag[count].value;
			    break;
			  case 28:
			  case 29:
			    camcnf->pv_cam.con1.b.iscale =
			      icp_cnf_flag[count].value;
			    break;
			  case 30:
			  case 31:
			    camcnf->pv_cam.con1.b.vclkpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 32:
			  case 33:
			    camcnf->pv_cam.con1.b.smode =
			      icp_cnf_flag[count].value;
			    break;
			  case 34:
			  case 35:
			    camcnf->pv_cam.con1.b.fldshft =
			      icp_cnf_flag[count].value;
			    break;
			  case 36:
			  case 37:
			    camcnf->pv_cam.trigcon.b.trigen =
			      icp_cnf_flag[count].value;
			    break;
			  case 38:
			  case 39:
			    camcnf->pv_cam.trigcon.b.trigsrc =
			      icp_cnf_flag[count].value;
			    break;
			  case 40:
			  case 41:
			    camcnf->pv_cam.trigcon.b.trigpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 42:
			  case 43:
			    camcnf->pv_cam.trigcon.b.strbmd =
			      icp_cnf_flag[count].value;
			    break;
			  case 44:
			  case 45:
			    camcnf->pv_cam.trigcon.b.strben =
			      icp_cnf_flag[count].value;
			    break;
			  case 46:
			  case 47:
			    camcnf->pv_cam.trigcon.b.strbpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 48:
			  case 49:
			    camcnf->pv_cam.trigcon.b.skpfldmd =
			      icp_cnf_flag[count].value;
			    break;
			  case 50:
			  case 51:
			    camcnf->pv_cam.p_clamp.b.pclmpol =
			      icp_cnf_flag[count].value;
			    break;
			  case 52:
			  case 53:
			    camcnf->pv_cam.p_clamp.b.clmpsrc =
			      icp_cnf_flag[count].value;
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			default:
			  return -EICP_MODULE;
			}
		      }
		      count++;
		    }
		    if (!i_valid) {
		      return -EICP_IDENTIFIER;
		    }
		    i_valid = FALSE;
		    count = 0;
		  }
		}
	|	WORD PARAM_FOLLOWS WORD
		{ if (y_valid) {
		    while (*icp_cnf_enum[count].identifier != '\0') {
		      if (!strcmp (icp_cnf_enum[count].identifier, $1)) {
			i_valid = TRUE;
		        if (!strcmp (icp_cnf_enum[count].parameter, $3)) {
			  ICP_PRINTF ("\t\t%s: %s\n",
				      icp_cnf_enum_debug[count].reg_name,
				      icp_cnf_enum_debug[count].meaning);
			  p_valid = TRUE;
			  switch (module) {
			  case ICP_AMFA:
			    break;
			  case ICP_AMC1:
			    break;
			  case ICP_AMVS:
			    switch (count) {
			    case 0:
			    case 1:
			    case 2:
			    case 3:
			    case 4:
			    case 5:
			    case 6:
			    case 7:
			    case 8:
			    case 9:
			    case 10:
			    case 11:
			    case 12:
			    case 13:
			      camcnf->vs_cam.tsel = icp_cnf_enum[count].value;
			      break;
			    case 14:
			    case 15:
			    case 16:
			    case 17:
			      camcnf->vs_cam.lpfsel = icp_cnf_enum[count].value;
			      break;
			    case 18:
			    case 19:
			    case 20:
			    case 21:
			      camcnf->vs_cam.iscale = icp_cnf_enum[count].value;
			      break;
			    case 22:
			    case 23:
			    case 24:
			    case 25:
			      camcnf->vs_cam.tsrc = icp_cnf_enum[count].value;
			      break;
			    case 26:
			    case 27:
			      camcnf->vs_cam.trgsrc = icp_cnf_enum[count].value;
			      break;
			    case 28:
			    case 29:
			      camcnf->vs_cam.fmode = icp_cnf_enum[count].value;
			      break;
			    case 30:
			    case 31:
			    case 32:
			    case 33:
			      camcnf->vs_cam.tmode = icp_cnf_enum[count].value;
			      break;
			    case 34:
			    case 35:
			      camcnf->vs_cam.cpsrc = icp_cnf_enum[count].value;
			      break;
			    case 36:
			    case 37:
			      camcnf->vs_cam.strbsel = icp_cnf_enum[count].value;
			      break;
			    default:
			      return -EICP_OVERFLOW;
			    }
			    break;
			  case ICP_AMDG:
			    switch (count) {
			    case 0:
			    case 1:
			    case 2:
			    case 3:
			    case 4:
			      camcnf->dg_cam.extmd = icp_cnf_enum[count].value;
			      break;
			    case 5:
			    case 6:
			      camcnf->dg_cam.muxa = icp_cnf_enum[count].value;
			      break;
			    case 7:
			    case 8:
			      camcnf->dg_cam.muxb = icp_cnf_enum[count].value;
			      break;
			    case 9:
			    case 10:
			      camcnf->dg_cam.muxc = icp_cnf_enum[count].value;
			      break;
			    case 11:
			    case 12:
			      camcnf->dg_cam.trigsrc = icp_cnf_enum[count].value;
			      break;
			    default:
			      return -EICP_OVERFLOW;
			    }
			    break;
			  case ICP_AMPV:
			    switch (count) {
			    default:
			      return -EICP_OVERFLOW;
			    }
			    break;
			  default:
			    return -EICP_MODULE;
			  }
			}
		      }
		      count++;
		    }
		    if (!i_valid) {
		      return -EICP_IDENTIFIER;
		    }
		    if (!p_valid) {
		      return -EICP_ENUM_PARAMETER;
		    }
		    i_valid = p_valid = FALSE;
		    count = 0;
		  }
		}
	|	WORD PARAM_FOLLOWS DIGIT
		{ if (y_valid) {
		    while (*icp_cnf_num[count].identifier != '\0') {
		      if (!strcmp (icp_cnf_num[count].identifier, $1)) {
			i_valid = TRUE;
			ICP_PRINTF ("\t\t%s: %s= %d\n",
				    icp_cnf_num_debug[count].reg_name,
				    icp_cnf_num_debug[count].meaning, $3);
			switch (module) {
			case ICP_AMFA:
			  break;
			case ICP_AMC1:
			  break;
			case ICP_AMVS:
			  switch (count) {
			  case 0:
			    camcnf->vs_cam.strbdly = $3;
			    break;
			  case 1:
			    camcnf->vs_cam.hoff = $3;
			    break;
			  case 2:
			    camcnf->vs_cam.hsz = $3;
			    break;
			  case 3:
			    camcnf->vs_cam.voff = $3;
			    break;
			  case 4:
			    camcnf->vs_cam.vsz = $3;
			    break;
			  case 5:
			    camcnf->vs_cam.froff = $3;
			    break;
			  case 6:
			    camcnf->vs_cam.cppos = $3;
			    break;
			  case 7:
			    camcnf->vs_cam.nref =
			      (u_char)((float)$3/1200.0 * 256.0);
			    break;
			  case 8:
			    camcnf->vs_cam.pref =
			      (u_char)((float)$3/2000.0 * 256.0);
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			case ICP_AMDG:
			  switch (count) {
			  case 0:
			    camcnf->dg_cam.mc = $3;
			    break;
			  case 1:
			    camcnf->dg_cam.extime = $3;
			    break;
			  case 2:
			    camcnf->dg_cam.hoff = $3;
			    break;
			  case 3:
			    camcnf->dg_cam.hact = $3;
			    break;
			  case 4:
			    camcnf->dg_cam.voff = $3;
			    break;
			  case 5:
			    camcnf->dg_cam.vact = $3;
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			case ICP_AMPV:
			  switch (count) {
			  case 0:
			    camcnf->pv_cam.htotal = $3;
			    break;
			  case 1:
			    camcnf->pv_cam.hsyncpw = $3;
			    break;
			  case 2:
			    camcnf->pv_cam.hequipw = $3;
			    break;
			  case 3:
			    camcnf->pv_cam.hserrpw = $3;
			    break;
			  case 4:
			    camcnf->pv_cam.vtotal = $3;
			    break;
			  case 5:
			    camcnf->pv_cam.vsyncpw.b.vsyncpw = $3;
			    break;
			  case 6:
			    camcnf->pv_cam.vgstart = $3;
			    break;
			  case 7:
			    camcnf->pv_cam.vgwidth = $3;
			    break;
			  case 8:
			    camcnf->pv_cam.hoffset = $3;
			    break;
			  case 9:
			    camcnf->pv_cam.hactive = $3;
			    break;
			  case 10:
			    camcnf->pv_cam.voffset = $3;
			    break;
			  case 11:
			    camcnf->pv_cam.vactive = $3;
			    break;
			  case 12:
			    camcnf->pv_cam.strbdly.b.strbdly = $3;
			    break;
			  case 13:
			    camcnf->pv_cam.frstdly.b.froff = $3;
			    break;
			  case 14:
			    camcnf->pv_cam.p_clamp.b.ccnt = $3;
			    break;
			  case 15:
			    camcnf->pv_cam.pref =
			      (u_char)((float)$3/2000.0 * 256.0);
			    break;
			  case 16:
			    camcnf->pv_cam.nref =
			      (u_char)((float)$3/1200.0 * 256.0);
			    break;
			  case 17:
			    camcnf->pv_cam.fdiv = $3;
			    break;
			  case 18:
			    camcnf->pv_cam.rdiv = $3;
			    break;
			  case 19:
			    camcnf->pv_cam.vcogain = $3;
			    break;
			  case 20:
			    camcnf->pv_cam.pda = $3;
			    break;
			  case 21:
			    camcnf->pv_cam.lcnt = $3;
			    break;
			  default:
			    return -EICP_OVERFLOW;
			  }
			  break;
			default:
			  return -EICP_MODULE;
			}
		      }
		      count++;
		    }
		    if (!i_valid) {
		      return -EICP_IDENTIFIER;
		    }
		    i_valid = FALSE;
		    count = 0;
		  }
		}
	|	WORD PARAM_FOLLOWS STRING
		{ if (y_valid) {
		    if (!strcmp ("Camera_Name", $1)) {
		      i_valid = TRUE;
		      ICP_PRINTF ("\t\tCamera_Name: %s\n", $3);
		      strcpy (camera_name, $3);
		    }
		    if (!i_valid) {
		      return -EICP_IDENTIFIER;
		    }
		    i_valid = FALSE;
		  }
		}
	;

%%

int
read_config (const char *config_file,
	     union icp_dev_t *config_camcnf,
	     int config_board,
	     int config_module,
             int config_camera,
	     char *config_camera_name) {

  int	error;

  /* argument validation */

  if (config_board >= ICP_BOARDS_MAX) { 
    return -EICP_BOARD;
  }

  switch (config_module) {
    case ICP_AMVS:
#ifdef DEBUG_ICPCI
      assert (sizeof(vs_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(vs_cnf_flag_debug) /
	      sizeof(struct icp_cnf_flag_debug_t) + 1);
      assert (sizeof(vs_cnf_enum) /
	      sizeof(struct icp_cnf_enum_t) !=
	      sizeof(vs_cnf_enum_debug) /
	      sizeof(struct icp_cnf_enum_debug_t) + 1);
      assert (sizeof(vs_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(vs_cnf_num_debug) /
	      sizeof(struct icp_cnf_num_debug_t) + 1);
#endif
      if (config_camera >= VS_CAMERAS_MAX) {
	return -EICP_CAMERA;
      }
      break;
    case ICP_AMDG:
#ifdef DEBUG_ICPCI
      assert (sizeof(dg_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(dg_cnf_flag_debug) /
	      sizeof(struct icp_cnf_flag_debug_t) + 1);
      assert (sizeof(dg_cnf_enum) /
	      sizeof(struct icp_cnf_enum_t) !=
	      sizeof(dg_cnf_enum_debug) /
	      sizeof(struct icp_cnf_enum_debug_t) + 1);
      assert (sizeof(dg_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(dg_cnf_num_debug) /
	      sizeof(struct icp_cnf_num_debug_t) + 1);
#endif
      if (config_camera >= DG_CAMERAS_MAX) {
	return -EICP_CAMERA;
      }
      break;
    case ICP_AMPV:
#ifdef DEBUG_ICPCI
      assert (sizeof(pv_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(pv_cnf_flag_debug) /
	      sizeof(struct icp_cnf_flag_debug_t) + 1);
      assert (sizeof(pv_cnf_enum) /
	      sizeof(struct icp_cnf_enum_t) !=
	      sizeof(pv_cnf_enum_debug) /
	      sizeof(struct icp_cnf_enum_debug_t) + 1);
      assert (sizeof(pv_cnf_flag) /
	      sizeof(struct icp_cnf_flag_t) !=
	      sizeof(pv_cnf_num_debug) /
	      sizeof(struct icp_cnf_num_debug_t) + 1);
#endif
      if (config_camera >= PV_CAMERAS_MAX) {
	return -EICP_CAMERA;
      }
      break;
    default:
      return -EICP_MODULE;
  }

  if ((yyin = fopen (config_file, "r" )) == NULL) {
    return -EICP_FILE;
  }

  camcnf = config_camcnf;
  board = config_board;
  module = config_module;
  camera = config_camera;

#ifdef YYDEBUG
  yydebug = 1;
#endif

  while (!feof(yyin)) {
    if ((error = yyparse()) < 0) {
      return (error);
    }
  }

  /* error checking */

  if (!readout) {
    if (!boardfound) {
      readout = modulefound = camerafound = FALSE;
      return -EICP_BOARD;
    } else {
      if (!modulefound) {
	readout = boardfound = camerafound = FALSE;
	return -EICP_MODULE;
      } else {
	if (!camerafound) {
	  readout = boardfound = modulefound = FALSE;
	  return -EICP_CAMERA;
	}
      }
    }
  }

  if (config_camera_name != NULL) {
    memset (config_camera_name, '\0', MAX_ENTRY_LENGTH);
    if (strlen (camera_name) > 2) {
      strncpy (config_camera_name, camera_name + 1, strlen (camera_name) - 2);
    }
  }

  readout = boardfound = modulefound = camerafound = FALSE;

  fclose (yyin);

  return (OK);
}

int
test_config (const char* config_file,
	     int *ret_boards,
	     int *ret_module, 
	     int ret_camera[ICP_BOARDS_MAX][ICP_CAMERAS_MAX]) {

  int error;

  int board, module, camera;
  
  int board_ok = FALSE,
      module_ok = FALSE;

  union icp_dev_t *cam_cnf;

  if ((cam_cnf = (union icp_dev_t*)malloc (sizeof(union icp_dev_t))) == NULL) {
    return (ERROR);
  }

  if (ret_boards != NULL) {
    *ret_boards = 0;
  }
  if (ret_module != NULL) {
    memset (ret_module, ICP_AMNO, ICP_BOARDS_MAX);
  }
  if (ret_camera != NULL) {
    memset (ret_camera, FALSE, ICP_BOARDS_MAX * ICP_CAMERAS_MAX);
  }

  board = -1;

  next_board:

  board++;
  board_ok = FALSE;
  if (board <= ICP_BOARDS_MAX) {
    module = ICP_AMNO;

  next_module:

    module++;
    module_ok = FALSE;
    if (module <= ICP_MODULE_MAX) {
      for (camera = 0; camera < ICP_CAMERAS_MAX; camera++) {
	error = read_config (config_file, cam_cnf,
			     board, module, camera, NULL);
	switch (error) {
	case -EICP_FILE:
	  free (cam_cnf);
	  return -EICP_FILE;
	case -EICP_BOARD: 
	  goto next_board;
	case -EICP_MODULE:
	  goto next_module;
	case -EICP_CAMERA:
	  break;
	case -EICP_OVERFLOW:
	case -EICP_IDENTIFIER:
	case -EICP_ENUM_PARAMETER:
	case -EICP_NUM_PARAMETER:
	  break;
	case -EICP_NOT_SPECIFIED:
	  free (cam_cnf);	  
	  return -EICP_NOT_SPECIFIED;
	default:
	  if (!board_ok) {
	    if (ret_boards != NULL) {
	      (*ret_boards)++;
	    }
	    board_ok = TRUE;
	  }
	  if (!module_ok) {
	    if (ret_module != NULL) {
	      ret_module[board] = module;
	    }
	    module_ok = TRUE;
	  }
	  if (ret_camera != NULL) {
	    ret_camera[board][camera] = TRUE;
	  }
	}
      }
      if (module_ok) {
	goto next_board;
      }
      goto next_module;
    } else {
      goto next_board;
    }
  }
  free (cam_cnf);
  return (OK);
}


int
yyerror (char *s) {

  ICP_PRINTF("\t\terror  : %s\n", s);

}

